@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2018 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -8,15 +8,25 @@
 
 """
 
-from .interfaces import MapperOption, PropComparator, MapperProperty
-from .attributes import QueryableAttribute
-from .. import util
-from ..sql.base import _generative, Generative
-from .. import exc as sa_exc, inspect
-from .base import _is_aliased_class, _class_to_mapper, _is_mapped_class
 from . import util as orm_util
-from .path_registry import PathRegistry, TokenRegistry, \
-    _WILDCARD_TOKEN, _DEFAULT_TOKEN
+from .attributes import QueryableAttribute
+from .base import _class_to_mapper
+from .base import _is_aliased_class
+from .base import _is_mapped_class
+from .base import InspectionAttr
+from .interfaces import MapperOption
+from .interfaces import PropComparator
+from .path_registry import _DEFAULT_TOKEN
+from .path_registry import _WILDCARD_TOKEN
+from .path_registry import PathRegistry
+from .path_registry import TokenRegistry
+from .util import _orm_full_deannotate
+from .. import exc as sa_exc
+from .. import inspect
+from .. import util
+from ..sql import expression as sql_expr
+from ..sql.base import _generative
+from ..sql.base import Generative
 
 
 class Load(Generative, MapperOption):
@@ -83,50 +93,62 @@ class Load(Generative, MapperOption):
             if key != "loader":
                 continue
 
-            endpoint = obj._of_type or obj.path.path[-1]
-            chopped = self._chop_path(loader_path, path)
+            for local_elem, obj_elem in zip(self.path.path, loader_path):
+                if local_elem is not obj_elem:
+                    break
+            else:
+                endpoint = obj._of_type or obj.path.path[-1]
+                chopped = self._chop_path(loader_path, path)
 
-            if (
-                # means loader_path and path are unrelated,
-                # this does not need to be part of a cache key
-                chopped is None
-            ) or (
-                # means no additional path with loader_path + path
-                # and the endpoint isn't using of_type so isn't modified into
-                # an alias or other unsafe entity
-                not chopped and not obj._of_type
-            ):
-                continue
+                if (
+                    # means loader_path and path are unrelated,
+                    # this does not need to be part of a cache key
+                    chopped
+                    is None
+                ) or (
+                    # means no additional path with loader_path + path
+                    # and the endpoint isn't using of_type so isn't modified
+                    # into an alias or other unsafe entity
+                    not chopped
+                    and not obj._of_type
+                ):
+                    continue
 
-            serialized_path = []
+                serialized_path = []
 
-            for token in chopped:
-                if isinstance(token, util.string_types):
-                    serialized_path.append(token)
-                elif token.is_aliased_class:
-                    return False
-                elif token.is_property:
-                    serialized_path.append(token.key)
-                else:
-                    assert token.is_mapper
-                    serialized_path.append(token.class_)
+                for token in chopped:
+                    if isinstance(token, util.string_types):
+                        serialized_path.append(token)
+                    elif token.is_aliased_class:
+                        return False
+                    elif token.is_property:
+                        serialized_path.append(token.key)
+                    else:
+                        assert token.is_mapper
+                        serialized_path.append(token.class_)
 
-            if not serialized_path or endpoint != serialized_path[-1]:
-                if endpoint.is_mapper:
-                    serialized_path.append(endpoint.class_)
-                elif endpoint.is_aliased_class:
-                    return False
+                if not serialized_path or endpoint != serialized_path[-1]:
+                    if endpoint.is_mapper:
+                        serialized_path.append(endpoint.class_)
+                    elif endpoint.is_aliased_class:
+                        return False
 
-            serialized.append(
-                (
-                    tuple(serialized_path) +
-                    (obj.strategy or ()) +
-                    (tuple([
-                        (key, obj.local_opts[key])
-                        for key in sorted(obj.local_opts)
-                    ]) if obj.local_opts else ())
+                serialized.append(
+                    (
+                        tuple(serialized_path)
+                        + (obj.strategy or ())
+                        + (
+                            tuple(
+                                [
+                                    (key, obj.local_opts[key])
+                                    for key in sorted(obj.local_opts)
+                                ]
+                            )
+                            if obj.local_opts
+                            else ()
+                        )
+                    )
                 )
-            )
         if not serialized:
             return None
         else:
@@ -158,18 +180,20 @@ class Load(Generative, MapperOption):
         else:
             query._attributes.update(self.context)
 
-    def _generate_path(self, path, attr, wildcard_key, raiseerr=True):
+    def _generate_path(
+        self, path, attr, for_strategy, wildcard_key, raiseerr=True
+    ):
+        existing_of_type = self._of_type
         self._of_type = None
-
         if raiseerr and not path.has_entity:
             if isinstance(path, TokenRegistry):
                 raise sa_exc.ArgumentError(
-                    "Wildcard token cannot be followed by another entity")
+                    "Wildcard token cannot be followed by another entity"
+                )
             else:
                 raise sa_exc.ArgumentError(
-                    "Attribute '%s' of entity '%s' does not "
-                    "refer to a mapped entity" %
-                    (path.prop.key, path.parent.entity)
+                    "Mapped attribute '%s' does not "
+                    "refer to a mapped entity" % (path.prop,)
                 )
 
         if isinstance(attr, util.string_types):
@@ -179,98 +203,100 @@ class Load(Generative, MapperOption):
                     self.propagate_to_loaders = False
                 if wildcard_key:
                     attr = "%s:%s" % (wildcard_key, attr)
+
+                # TODO: AliasedInsp inside the path for of_type is not
+                # working for a with_polymorphic entity because the
+                # relationship loaders don't render the with_poly into the
+                # path.  See #4469 which will try to improve this
+                if existing_of_type and not existing_of_type.is_aliased_class:
+                    path = path.parent[existing_of_type]
                 path = path.token(attr)
                 self.path = path
                 return path
 
+            if existing_of_type:
+                ent = inspect(existing_of_type)
+            else:
+                ent = path.entity
+
             try:
                 # use getattr on the class to work around
                 # synonyms, hybrids, etc.
-                attr = getattr(path.entity.class_, attr)
+                attr = getattr(ent.class_, attr)
             except AttributeError:
                 if raiseerr:
                     raise sa_exc.ArgumentError(
-                        "Can't find property named '%s' on the "
-                        "mapped entity %s in this Query. " % (
-                            attr, path.entity)
+                        'Can\'t find property named "%s" on '
+                        "%s in this Query. " % (attr, ent)
                     )
                 else:
                     return None
             else:
-                attr = attr.property
+                attr = found_property = attr.property
 
             path = path[attr]
         elif _is_mapped_class(attr):
-            if not attr.common_parent(path.mapper):
+            if not orm_util._entity_corresponds_to_use_path_impl(
+                attr.parent, path[-1]
+            ):
                 if raiseerr:
                     raise sa_exc.ArgumentError(
                         "Attribute '%s' does not "
-                        "link from element '%s'" % (attr, path.entity))
+                        "link from element '%s'" % (attr, path.entity)
+                    )
                 else:
                     return None
         else:
-            prop = attr.property
+            prop = found_property = attr.property
 
-            if not prop.parent.common_parent(path.mapper):
+            if not orm_util._entity_corresponds_to_use_path_impl(
+                attr.parent, path[-1]
+            ):
                 if raiseerr:
                     raise sa_exc.ArgumentError(
-                        "Attribute '%s' does not "
-                        "link from element '%s'" % (attr, path.entity))
+                        'Attribute "%s" does not '
+                        'link from element "%s"' % (attr, path.entity)
+                    )
                 else:
                     return None
 
-            if getattr(attr, '_of_type', None):
+            if getattr(attr, "_of_type", None):
                 ac = attr._of_type
                 ext_info = of_type_info = inspect(ac)
 
                 existing = path.entity_path[prop].get(
-                    self.context, "path_with_polymorphic")
+                    self.context, "path_with_polymorphic"
+                )
                 if not ext_info.is_aliased_class:
                     ac = orm_util.with_polymorphic(
                         ext_info.mapper.base_mapper,
-                        ext_info.mapper, aliased=True,
+                        ext_info.mapper,
+                        aliased=True,
                         _use_mapper_path=True,
-                        _existing_alias=existing)
-                    ext_info = inspect(ac)
-                elif not ext_info.with_polymorphic_mappers:
-                    ext_info = orm_util.AliasedInsp(
-                        ext_info.entity,
-                        ext_info.mapper.base_mapper,
-                        ext_info.selectable,
-                        ext_info.name,
-                        ext_info.with_polymorphic_mappers or [ext_info.mapper],
-                        ext_info.polymorphic_on,
-                        ext_info._base_alias,
-                        ext_info._use_mapper_path,
-                        ext_info._adapt_on_names,
-                        ext_info.represents_outer_join
+                        _existing_alias=existing,
                     )
+                    ext_info = inspect(ac)
 
                 path.entity_path[prop].set(
-                    self.context, "path_with_polymorphic", ext_info)
+                    self.context, "path_with_polymorphic", ext_info
+                )
 
-                # the path here will go into the context dictionary and
-                # needs to match up to how the class graph is traversed.
-                # so we can't put an AliasedInsp in the path here, needs
-                # to be the base mapper.
-                path = path[prop][ext_info.mapper]
+                path = path[prop][ext_info]
 
-                # but, we need to know what the original of_type()
-                # argument is for cache key purposes.  so....store that too.
-                # it might be better for "path" to really represent,
-                # "the path", but trying to keep the impact of the cache
-                # key feature localized for now
                 self._of_type = of_type_info
+
             else:
                 path = path[prop]
 
+        if for_strategy is not None:
+            found_property._get_strategy(for_strategy)
         if path.has_entity:
             path = path.entity_path
         self.path = path
         return path
 
     def __str__(self):
-        return "Load(strategy=%r)" % (self.strategy, )
+        return "Load(strategy=%r)" % (self.strategy,)
 
     def _coerce_strat(self, strategy):
         if strategy is not None:
@@ -279,13 +305,14 @@ class Load(Generative, MapperOption):
 
     @_generative
     def set_relationship_strategy(
-            self, attr, strategy, propagate_to_loaders=True):
+        self, attr, strategy, propagate_to_loaders=True
+    ):
         strategy = self._coerce_strat(strategy)
 
         self.is_class_strategy = False
         self.propagate_to_loaders = propagate_to_loaders
         # if the path is a wildcard, this will set propagate_to_loaders=False
-        self._generate_path(self.path, attr, "relationship")
+        self._generate_path(self.path, attr, strategy, "relationship")
         self.strategy = strategy
         if strategy is not None:
             self._set_path_strategy()
@@ -298,7 +325,7 @@ class Load(Generative, MapperOption):
         for attr in attrs:
             cloned = self._generate()
             cloned.strategy = strategy
-            cloned._generate_path(self.path, attr, "column")
+            cloned._generate_path(self.path, attr, strategy, "column")
             cloned.propagate_to_loaders = True
             if opts:
                 cloned.local_opts.update(opts)
@@ -312,7 +339,7 @@ class Load(Generative, MapperOption):
         strategy = self._coerce_strat(strategy)
 
         for attr in attrs:
-            path = self._generate_path(self.path, attr, None)
+            path = self._generate_path(self.path, attr, strategy, None)
             cloned = self._generate()
             cloned.strategy = strategy
             cloned.path = path
@@ -324,7 +351,7 @@ class Load(Generative, MapperOption):
         strategy = self._coerce_strat(strategy)
         cloned = self._generate()
         cloned.is_class_strategy = True
-        path = cloned._generate_path(self.path, None, None)
+        path = cloned._generate_path(self.path, None, strategy, None)
         cloned.strategy = strategy
         cloned.path = path
         cloned.propagate_to_loaders = True
@@ -352,9 +379,21 @@ class Load(Generative, MapperOption):
         else:
             effective_path = self.path
 
-        self._set_for_path(
-            self.context, effective_path, replace=True,
-            merge_opts=self.is_opts_only)
+        if effective_path.is_token:
+            for path in effective_path.generate_for_superclasses():
+                self._set_for_path(
+                    self.context,
+                    path,
+                    replace=True,
+                    merge_opts=self.is_opts_only,
+                )
+        else:
+            self._set_for_path(
+                self.context,
+                effective_path,
+                replace=True,
+                merge_opts=self.is_opts_only,
+            )
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -373,17 +412,26 @@ class Load(Generative, MapperOption):
                 # TODO: this is approximated from the _UnboundLoad
                 # version and probably has issues, not fully covered.
 
-                if i == 0 and c_token.endswith(':' + _DEFAULT_TOKEN):
+                if i == 0 and c_token.endswith(":" + _DEFAULT_TOKEN):
                     return to_chop
-                elif c_token != 'relationship:%s' % (_WILDCARD_TOKEN,) and \
-                        c_token != p_token.key:
+                elif (
+                    c_token != "relationship:%s" % (_WILDCARD_TOKEN,)
+                    and c_token != p_token.key
+                ):
                     return None
 
             if c_token is p_token:
                 continue
+            elif (
+                isinstance(c_token, InspectionAttr)
+                and c_token.is_mapper
+                and p_token.is_mapper
+                and c_token.isa(p_token)
+            ):
+                continue
             else:
                 return None
-        return to_chop[i + 1:]
+        return to_chop[i + 1 :]
 
 
 class _UnboundLoad(Load):
@@ -407,15 +455,17 @@ class _UnboundLoad(Load):
     def _generate_cache_key(self, path):
         serialized = ()
         for val in self._to_bind:
-            opt = val._bind_loader(
-                [path.path[0]],
-                None, None, False)
-            if opt:
-                c_key = opt._generate_cache_key(path)
-                if c_key is False:
-                    return False
-                elif c_key:
-                    serialized += c_key
+            for local_elem, val_elem in zip(self.path, val.path):
+                if local_elem is not val_elem:
+                    break
+            else:
+                opt = val._bind_loader([path.path[0]], None, None, False)
+                if opt:
+                    c_key = opt._generate_cache_key(path)
+                    if c_key is False:
+                        return False
+                    elif c_key:
+                        serialized += c_key
         if not serialized:
             return None
         else:
@@ -424,27 +474,30 @@ class _UnboundLoad(Load):
     def _set_path_strategy(self):
         self._to_bind.append(self)
 
-    def _generate_path(self, path, attr, wildcard_key):
-        if wildcard_key and isinstance(attr, util.string_types) and \
-                attr in (_WILDCARD_TOKEN, _DEFAULT_TOKEN):
+    def _generate_path(self, path, attr, for_strategy, wildcard_key):
+        if (
+            wildcard_key
+            and isinstance(attr, util.string_types)
+            and attr in (_WILDCARD_TOKEN, _DEFAULT_TOKEN)
+        ):
             if attr == _DEFAULT_TOKEN:
                 self.propagate_to_loaders = False
             attr = "%s:%s" % (wildcard_key, attr)
         if path and _is_mapped_class(path[-1]) and not self.is_class_strategy:
             path = path[0:-1]
         if attr:
-            path = path + (attr, )
+            path = path + (attr,)
         self.path = path
         return path
 
     def __getstate__(self):
         d = self.__dict__.copy()
-        d['path'] = self._serialize_path(self.path)
+        d["path"] = self._serialize_path(self.path, filter_aliased_class=True)
         return d
 
     def __setstate__(self, state):
         ret = []
-        for key in state['path']:
+        for key in state["path"]:
             if isinstance(key, tuple):
                 if len(key) == 2:
                     # support legacy
@@ -454,18 +507,24 @@ class _UnboundLoad(Load):
                     cls, propkey, of_type = key
                 prop = getattr(cls, propkey)
                 if of_type:
-                    prop = prop.of_type(prop)
+                    prop = prop.of_type(of_type)
                 ret.append(prop)
             else:
                 ret.append(key)
-        state['path'] = tuple(ret)
+        state["path"] = tuple(ret)
         self.__dict__ = state
 
     def _process(self, query, raiseerr):
+        dedupes = query._attributes["_unbound_load_dedupes"]
         for val in self._to_bind:
-            val._bind_loader(
-                [ent.entity_zero for ent in query._mapper_entities],
-                query._current_path, query._attributes, raiseerr)
+            if val not in dedupes:
+                dedupes.add(val)
+                val._bind_loader(
+                    [ent.entity_zero for ent in query._mapper_entities],
+                    query._current_path,
+                    query._attributes,
+                    raiseerr,
+                )
 
     @classmethod
     def _from_keys(cls, meth, keys, chained, kw):
@@ -475,13 +534,14 @@ class _UnboundLoad(Load):
             if isinstance(key, util.string_types):
                 # coerce fooload('*') into "default loader strategy"
                 if key == _WILDCARD_TOKEN:
-                    return (_DEFAULT_TOKEN, )
+                    return (_DEFAULT_TOKEN,)
                 # coerce fooload(".*") into "wildcard on default entity"
                 elif key.startswith("." + _WILDCARD_TOKEN):
                     key = key[1:]
                 return key.split(".")
             else:
                 return (key,)
+
         all_tokens = [token for key in keys for token in _split_key(key)]
 
         for token in all_tokens[0:-1]:
@@ -499,40 +559,44 @@ class _UnboundLoad(Load):
     def _chop_path(self, to_chop, path):
         i = -1
         for i, (c_token, (p_entity, p_prop)) in enumerate(
-                zip(to_chop, path.pairs())):
+            zip(to_chop, path.pairs())
+        ):
             if isinstance(c_token, util.string_types):
-                if i == 0 and c_token.endswith(':' + _DEFAULT_TOKEN):
+                if i == 0 and c_token.endswith(":" + _DEFAULT_TOKEN):
                     return to_chop
-                elif c_token != 'relationship:%s' % (
-                        _WILDCARD_TOKEN,) and c_token != p_prop.key:
+                elif (
+                    c_token != "relationship:%s" % (_WILDCARD_TOKEN,)
+                    and c_token != p_prop.key
+                ):
                     return None
             elif isinstance(c_token, PropComparator):
-                if c_token.property is not p_prop or \
-                        (
-                            c_token._parententity is not p_entity and (
-                                not c_token._parententity.is_mapper or
-                                not c_token._parententity.isa(p_entity)
-                            )
-                        ):
+                if c_token.property is not p_prop or (
+                    c_token._parententity is not p_entity
+                    and (
+                        not c_token._parententity.is_mapper
+                        or not c_token._parententity.isa(p_entity)
+                    )
+                ):
                     return None
         else:
             i += 1
 
         return to_chop[i:]
 
-    def _serialize_path(self, path, reject_aliased_class=False):
+    def _serialize_path(self, path, filter_aliased_class=False):
         ret = []
         for token in path:
             if isinstance(token, QueryableAttribute):
-                if reject_aliased_class and (
-                    (token._of_type and
-                        inspect(token._of_type).is_aliased_class)
-                    or
-                    inspect(token.parent).is_aliased_class
+                if (
+                    filter_aliased_class
+                    and token._of_type
+                    and inspect(token._of_type).is_aliased_class
                 ):
-                    return False
-                ret.append(
-                    (token._parentmapper.class_, token.key, token._of_type))
+                    ret.append((token._parentmapper.class_, token.key, None))
+                else:
+                    ret.append(
+                        (token._parentmapper.class_, token.key, token._of_type)
+                    )
             elif isinstance(token, PropComparator):
                 ret.append((token._parentmapper.class_, token.key, None))
             else:
@@ -578,7 +642,7 @@ class _UnboundLoad(Load):
         start_path = self.path
 
         if self.is_class_strategy and current_path:
-            start_path += (entities[0], )
+            start_path += (entities[0],)
 
         # _current_path implies we're in a
         # secondary load with an existing path
@@ -594,23 +658,20 @@ class _UnboundLoad(Load):
         token = start_path[0]
 
         if isinstance(token, util.string_types):
-            entity = self._find_entity_basestring(
-                entities, token, raiseerr)
+            entity = self._find_entity_basestring(entities, token, raiseerr)
         elif isinstance(token, PropComparator):
             prop = token.property
             entity = self._find_entity_prop_comparator(
-                entities,
-                prop.key,
-                token._parententity,
-                raiseerr)
+                entities, prop, token._parententity, raiseerr
+            )
         elif self.is_class_strategy and _is_mapped_class(token):
             entity = inspect(token)
             if entity not in entities:
                 entity = None
         else:
             raise sa_exc.ArgumentError(
-                "mapper option expects "
-                "string key or list of attributes")
+                "mapper option expects " "string key or list of attributes"
+            )
 
         if not entity:
             return
@@ -634,9 +695,14 @@ class _UnboundLoad(Load):
         path = loader.path
 
         if not loader.is_class_strategy:
-            for token in start_path:
+            for idx, token in enumerate(start_path):
                 if not loader._generate_path(
-                        loader.path, token, None, raiseerr):
+                    loader.path,
+                    token,
+                    self.strategy if idx == len(start_path) - 1 else None,
+                    None,
+                    raiseerr,
+                ):
                     return
 
         loader.local_opts.update(self.local_opts)
@@ -653,18 +719,22 @@ class _UnboundLoad(Load):
         if effective_path.is_token:
             for path in effective_path.generate_for_superclasses():
                 loader._set_for_path(
-                    context, path,
+                    context,
+                    path,
                     replace=not self._is_chain_link,
-                    merge_opts=self.is_opts_only)
+                    merge_opts=self.is_opts_only,
+                )
         else:
             loader._set_for_path(
-                context, effective_path,
+                context,
+                effective_path,
                 replace=not self._is_chain_link,
-                merge_opts=self.is_opts_only)
+                merge_opts=self.is_opts_only,
+            )
 
         return loader
 
-    def _find_entity_prop_comparator(self, entities, token, mapper, raiseerr):
+    def _find_entity_prop_comparator(self, entities, prop, mapper, raiseerr):
         if _is_aliased_class(mapper):
             searchfor = mapper
         else:
@@ -676,29 +746,39 @@ class _UnboundLoad(Load):
             if raiseerr:
                 if not list(entities):
                     raise sa_exc.ArgumentError(
-                        "Query has only expression-based entities - "
-                        "can't find property named '%s'."
-                        % (token, )
+                        "Query has only expression-based entities, "
+                        'which do not apply to %s "%s"'
+                        % (util.clsname_as_plain_name(type(prop)), prop)
                     )
                 else:
                     raise sa_exc.ArgumentError(
-                        "Can't find property '%s' on any entity "
-                        "specified in this Query.  Note the full path "
-                        "from root (%s) to target entity must be specified."
-                        % (token, ",".join(str(x) for
-                                           x in entities))
+                        'Mapped attribute "%s" does not apply to any of the '
+                        "root entities in this query, e.g. %s. Please "
+                        "specify the full path "
+                        "from one of the root entities to the target "
+                        "attribute. "
+                        % (prop, ", ".join(str(x) for x in entities))
                     )
             else:
                 return None
 
     def _find_entity_basestring(self, entities, token, raiseerr):
-        if token.endswith(':' + _WILDCARD_TOKEN):
+        if token.endswith(":" + _WILDCARD_TOKEN):
             if len(list(entities)) != 1:
                 if raiseerr:
                     raise sa_exc.ArgumentError(
-                        "Wildcard loader can only be used with exactly "
-                        "one entity.  Use Load(ent) to specify "
-                        "specific entities.")
+                        "Can't apply wildcard ('*') or load_only() "
+                        "loader option to multiple entities %s. Specify "
+                        "loader options for each entity individually, such "
+                        "as %s."
+                        % (
+                            ", ".join(str(ent) for ent in entities),
+                            ", ".join(
+                                "Load(%s).some_option('*')" % ent
+                                for ent in entities
+                            ),
+                        )
+                    )
         elif token.endswith(_DEFAULT_TOKEN):
             raiseerr = False
 
@@ -711,8 +791,7 @@ class _UnboundLoad(Load):
             if raiseerr:
                 raise sa_exc.ArgumentError(
                     "Query has only expression-based entities - "
-                    "can't find property named '%s'."
-                    % (token, )
+                    'can\'t find property named "%s".' % (token,)
                 )
             else:
                 return None
@@ -739,24 +818,38 @@ class loader_option(object):
 
 See :func:`.orm.%(name)s` for usage examples.
 
-""" % {"name": self.name}
+""" % {
+            "name": self.name
+        }
 
         fn.__doc__ = fn_doc
         return self
 
     def _add_unbound_all_fn(self, fn):
-        self._unbound_all_fn = fn
         fn.__doc__ = """Produce a standalone "all" option for :func:`.orm.%(name)s`.
 
-.. deprecated:: 0.9.0
+.. deprecated:: 0.9
 
-    The "_all()" style is replaced by method chaining, e.g.::
+    The :func:`.%(name)s_all` function is deprecated, and will be removed
+    in a future release.  Please use method chaining with :func:`.%(name)s`
+    instead, as in::
 
         session.query(MyClass).options(
             %(name)s("someattribute").%(name)s("anotherattribute")
         )
 
-""" % {"name": self.name}
+""" % {
+            "name": self.name
+        }
+        fn = util.deprecated(
+            "0.9",
+            "The :func:`.%(name)s_all` function is deprecated, and will be "
+            "removed in a future release.  Please use method chaining with "
+            ":func:`.%(name)s` instead" % {"name": self.name},
+            add_deprecation_to_docstring=False,
+        )(fn)
+
+        self._unbound_all_fn = fn
         return self
 
 
@@ -813,23 +906,22 @@ def contains_eager(loadopt, attr, alias=None):
             info = inspect(alias)
             alias = info.selectable
 
-    elif getattr(attr, '_of_type', None):
+    elif getattr(attr, "_of_type", None):
         ot = inspect(attr._of_type)
         alias = ot.selectable
 
     cloned = loadopt.set_relationship_strategy(
-        attr,
-        {"lazy": "joined"},
-        propagate_to_loaders=False
+        attr, {"lazy": "joined"}, propagate_to_loaders=False
     )
-    cloned.local_opts['eager_from_alias'] = alias
+    cloned.local_opts["eager_from_alias"] = alias
     return cloned
 
 
 @contains_eager._add_unbound_fn
 def contains_eager(*keys, **kw):
     return _UnboundLoad()._from_keys(
-        _UnboundLoad.contains_eager, keys, True, kw)
+        _UnboundLoad.contains_eager, keys, True, kw
+    )
 
 
 @loader_option()
@@ -867,12 +959,11 @@ def load_only(loadopt, *attrs):
 
     """
     cloned = loadopt.set_column_strategy(
-        attrs,
-        {"deferred": False, "instrument": True}
+        attrs, {"deferred": False, "instrument": True}
+    )
+    cloned.set_column_strategy(
+        "*", {"deferred": True, "instrument": True}, {"undefer_pks": True}
     )
-    cloned.set_column_strategy("*",
-                               {"deferred": True, "instrument": True},
-                               {"undefer_pks": True})
     return cloned
 
 
@@ -969,20 +1060,18 @@ def joinedload(loadopt, attr, innerjoin=None):
     """
     loader = loadopt.set_relationship_strategy(attr, {"lazy": "joined"})
     if innerjoin is not None:
-        loader.local_opts['innerjoin'] = innerjoin
+        loader.local_opts["innerjoin"] = innerjoin
     return loader
 
 
 @joinedload._add_unbound_fn
 def joinedload(*keys, **kw):
-    return _UnboundLoad._from_keys(
-        _UnboundLoad.joinedload, keys, False, kw)
+    return _UnboundLoad._from_keys(_UnboundLoad.joinedload, keys, False, kw)
 
 
 @joinedload._add_unbound_all_fn
 def joinedload_all(*keys, **kw):
-    return _UnboundLoad._from_keys(
-        _UnboundLoad.joinedload, keys, True, kw)
+    return _UnboundLoad._from_keys(_UnboundLoad.joinedload, keys, True, kw)
 
 
 @loader_option()
@@ -1125,8 +1214,7 @@ def immediateload(loadopt, attr):
 
 @immediateload._add_unbound_fn
 def immediateload(*keys):
-    return _UnboundLoad._from_keys(
-        _UnboundLoad.immediateload, keys, False, {})
+    return _UnboundLoad._from_keys(_UnboundLoad.immediateload, keys, False, {})
 
 
 @loader_option()
@@ -1186,7 +1274,8 @@ def raiseload(loadopt, attr, sql_only=False):
     """
 
     return loadopt.set_relationship_strategy(
-        attr, {"lazy": "raise_on_sql" if sql_only else "raise"})
+        attr, {"lazy": "raise_on_sql" if sql_only else "raise"}
+    )
 
 
 @raiseload._add_unbound_fn
@@ -1224,10 +1313,7 @@ def defaultload(loadopt, attr):
         :ref:`deferred_loading_w_multiple`
 
     """
-    return loadopt.set_relationship_strategy(
-        attr,
-        None
-    )
+    return loadopt.set_relationship_strategy(attr, None)
 
 
 @defaultload._add_unbound_fn
@@ -1237,8 +1323,8 @@ def defaultload(*keys):
 
 @loader_option()
 def defer(loadopt, key):
-    r"""Indicate that the given column-oriented attribute should be deferred, e.g.
-    not loaded until accessed.
+    r"""Indicate that the given column-oriented attribute should be deferred,
+    e.g. not loaded until accessed.
 
     This function is part of the :class:`.Load` interface and supports
     both method-chained and standalone operation.
@@ -1276,10 +1362,16 @@ def defer(loadopt, key):
 
     :param key: Attribute to be deferred.
 
-    :param \*addl_attrs: Deprecated; this option supports the old 0.8 style
+    :param \*addl_attrs: This option supports the old 0.8 style
      of specifying a path as a series of attributes, which is now superseded
      by the method-chained style.
 
+        .. deprecated:: 0.9  The \*addl_attrs on :func:`.orm.defer` is
+           deprecated and will be removed in a future release.   Please
+           use method chaining in conjunction with defaultload() to
+           indicate a path.
+
+
     .. seealso::
 
         :ref:`deferred`
@@ -1288,15 +1380,21 @@ def defer(loadopt, key):
 
     """
     return loadopt.set_column_strategy(
-        (key, ),
-        {"deferred": True, "instrument": True}
+        (key,), {"deferred": True, "instrument": True}
     )
 
 
 @defer._add_unbound_fn
 def defer(key, *addl_attrs):
+    if addl_attrs:
+        util.warn_deprecated(
+            "The *addl_attrs on orm.defer is deprecated.  Please use "
+            "method chaining in conjunction with defaultload() to "
+            "indicate a path."
+        )
     return _UnboundLoad._from_keys(
-        _UnboundLoad.defer, (key, ) + addl_attrs, False, {})
+        _UnboundLoad.defer, (key,) + addl_attrs, False, {}
+    )
 
 
 @loader_option()
@@ -1319,12 +1417,21 @@ def undefer(loadopt, key):
         session.query(MyClass, MyOtherClass).options(
             Load(MyClass).undefer("*"))
 
+        # undefer a column on a related object
+        session.query(MyClass).options(
+            defaultload(MyClass.items).undefer('text'))
+
     :param key: Attribute to be undeferred.
 
-    :param \*addl_attrs: Deprecated; this option supports the old 0.8 style
+    :param \*addl_attrs: This option supports the old 0.8 style
      of specifying a path as a series of attributes, which is now superseded
      by the method-chained style.
 
+        .. deprecated:: 0.9  The \*addl_attrs on :func:`.orm.undefer` is
+           deprecated and will be removed in a future release.   Please
+           use method chaining in conjunction with defaultload() to
+           indicate a path.
+
     .. seealso::
 
         :ref:`deferred`
@@ -1335,15 +1442,21 @@ def undefer(loadopt, key):
 
     """
     return loadopt.set_column_strategy(
-        (key, ),
-        {"deferred": False, "instrument": True}
+        (key,), {"deferred": False, "instrument": True}
     )
 
 
 @undefer._add_unbound_fn
 def undefer(key, *addl_attrs):
+    if addl_attrs:
+        util.warn_deprecated(
+            "The *addl_attrs on orm.undefer is deprecated.  Please use "
+            "method chaining in conjunction with defaultload() to "
+            "indicate a path."
+        )
     return _UnboundLoad._from_keys(
-        _UnboundLoad.undefer, (key, ) + addl_attrs, False, {})
+        _UnboundLoad.undefer, (key,) + addl_attrs, False, {}
+    )
 
 
 @loader_option()
@@ -1366,7 +1479,7 @@ def undefer_group(loadopt, name):
             defaultload("someattr").undefer_group("large_attrs"))
 
     .. versionchanged:: 0.9.0 :func:`.orm.undefer_group` is now specific to a
-       particiular entity load path.
+       particular entity load path.
 
     .. seealso::
 
@@ -1378,10 +1491,7 @@ def undefer_group(loadopt, name):
 
     """
     return loadopt.set_column_strategy(
-        "*",
-        None,
-        {"undefer_group_%s" % name: True},
-        opts_only=True
+        "*", None, {"undefer_group_%s" % name: True}, opts_only=True
     )
 
 
@@ -1390,10 +1500,6 @@ def undefer_group(name):
     return _UnboundLoad().undefer_group(name)
 
 
-from ..sql import expression as sql_expr
-from .util import _orm_full_deannotate
-
-
 @loader_option()
 def with_expression(loadopt, key, expression):
     r"""Apply an ad-hoc SQL expression to a "deferred expression" attribute.
@@ -1421,21 +1527,18 @@ def with_expression(loadopt, key, expression):
 
     """
 
-    expression = sql_expr._labeled(
-        _orm_full_deannotate(expression))
+    expression = sql_expr._labeled(_orm_full_deannotate(expression))
 
     return loadopt.set_column_strategy(
-        (key, ),
-        {"query_expression": True},
-        opts={"expression": expression}
+        (key,), {"query_expression": True}, opts={"expression": expression}
     )
 
 
 @with_expression._add_unbound_fn
 def with_expression(key, expression):
     return _UnboundLoad._from_keys(
-        _UnboundLoad.with_expression, (key, ),
-        False, {"expression": expression})
+        _UnboundLoad.with_expression, (key,), False, {"expression": expression}
+    )
 
 
 @loader_option()
@@ -1456,7 +1559,11 @@ def selectin_polymorphic(loadopt, classes):
     """
     loadopt.set_class_strategy(
         {"selectinload_polymorphic": True},
-        opts={"entities": tuple(sorted((inspect(cls) for cls in classes), key=id))}
+        opts={
+            "entities": tuple(
+                sorted((inspect(cls) for cls in classes), key=id)
+            )
+        },
     )
     return loadopt
 
@@ -1465,8 +1572,6 @@ def selectin_polymorphic(loadopt, classes):
 def selectin_polymorphic(base_cls, classes):
     ul = _UnboundLoad()
     ul.is_class_strategy = True
-    ul.path = (inspect(base_cls), )
-    ul.selectin_polymorphic(
-        classes
-    )
+    ul.path = (inspect(base_cls),)
+    ul.selectin_polymorphic(classes)
     return ul
